import React, { useRef, useState, useCallback } from 'react';
import { useEffectOnce } from './hooks/use-effect-once';
import "./styles/globals.css";
import type { Action, Position, TextHighlightPopupProps, ActionPayload } from './utils/types';

export function TextHighlightPopup<T = string>({
    children,
    actions,
    getActionPayload = (text) => text as ActionPayload<T>
}: TextHighlightPopupProps<T>) {
    const [isPopupVisible, setIsPopupVisible] = useState(false);
    const [popupPosition, setPopupPosition] = useState<Position>({ top: 0, left: 0 });
    const [selectedText, setSelectedText] = useState('');
    const containerRef = useRef<HTMLDivElement>(null);
    const popupRef = useRef<HTMLDivElement>(null);

    const calculatePopupPosition = useCallback((rect: DOMRect, containerRect: DOMRect): Position => {
        const popupWidth = popupRef.current?.offsetWidth || 0;
        const popupHeight = popupRef.current?.offsetHeight || 0;

        let top = Math.max(0, rect.bottom - containerRect.top);
        let left = rect.left - containerRect.left + rect.width / 2;

        if (left + popupWidth > containerRect.width) {
            left = containerRect.width - popupWidth;
        } else if (left < 0) {
            left = 10;
        }

        if (top + popupHeight > containerRect.height) {
            top = rect.top - containerRect.top - popupHeight;
        } else if (top < 0) {
            top = 10;
        }

        return { top, left };
    }, []);

    const handleMouseUp = useCallback(() => {
        const selection = window.getSelection();
        if (selection && selection.toString().trim().length > 0) {
            const range = selection.getRangeAt(0);
            const rect = range.getBoundingClientRect();
            const containerRect = containerRef.current?.getBoundingClientRect();

            if (containerRect) {
                const newPosition = calculatePopupPosition(rect, containerRect);
                setPopupPosition(newPosition);
            }

            setSelectedText(selection.toString().trim());
            setIsPopupVisible(true);
        }
    }, [calculatePopupPosition]);

    const handleActionClick = useCallback((action: Action<T>) => {
        const payload = getActionPayload(selectedText);
        if (action.action) {
            action.action(payload);
        } else null
        setIsPopupVisible(false);
    }, [getActionPayload, selectedText]);


    const handleSelectionChange = useCallback(() => {
        const selection = window.getSelection();
        if (!selection || selection.toString().trim().length === 0) {
            setIsPopupVisible(false);
        }
    }, []);

    useEffectOnce(() => {
        document.addEventListener('selectionchange', handleSelectionChange, { capture: true });
        return () => document.removeEventListener('selectionchange', handleSelectionChange);
    });

    return (
        <div
            ref={containerRef}
            className="text-highlight-container relative"
            onMouseUp={handleMouseUp}
        >
            {children}
            {isPopupVisible && (
                <div
                    ref={popupRef}
                    className="text-highlight-popup absolute transition popup"
                    style={{
                        top: `${popupPosition.top}px`,
                        left: `${popupPosition.left}px`,
                    }}
                >
                    <div className="flex space-x-2 bg-transparent p-2" aria-label="Text highlight actions">
                        {actions.slice(0, 3).map((action) => (
                            <div key={action.label} className="action-container bg-white rounded shadow flex flex-row items-center p-2">
                                <button
                                    type="button"
                                    onClick={() => handleActionClick(action)}
                                    className="button button-tertiary-grey button-sm flex-grow flex items-center justify-center"
                                    aria-label={action.label}
                                >
                                    {action.icon && <span className="button-icon mr-2" aria-hidden="true">{action.icon}</span>}
                                    <span className="button-text">{action.label}</span>
                                </button>
                            </div>
                        ))}
                    </div>
                </div>
            )}
        </div>
    );
}